@@ -4,6 +4,7 @@ import appcli
 import autoprop
 import stepwise
 import freezerbox
+import shlex
 
 from freezerbox import (
         load_maker_factory, group_by_synthesis, group_by_cleanup,
@@ -17,24 +18,31 @@ from inform import plural
 from os import getcwd
 from os.path import expanduser
 
-@autoprop
+@autoprop.cache
 class Make(appcli.App):
     """\
 Display a protocol for making the given reagents.
 
 Usage:
-    make <tags>... [-P]
+    make <tags>... [-R] [-x <tags>]
 
 Arguments:
     <tags>
-        The name of a reagent in the freezerbox database, e.g. p01 or f01.
+        The names of any number of reagents in the FreezerBox database, e.g. 
+        p01 or f01.  By default, any other reagents that are needed to make the 
+        named reagents and that are marked as "not ready" (e.g. "n", "no", "0" 
+        in the "Ready" column) will also be included in the resulting protocol.
 
 Options:
-
-    -P --no-pending
+    -R --no-recurse
         Only make the reagents specified on the command line; don't 
-        automatically include dependencies that are marked as "pending" in the 
-        database.
+        automatically include dependencies that are marked as "not ready" in 
+        the database.
+
+    -x --exclude <tags>
+        A comma-separated list of tags to exclude from the protocol, i.e. 
+        dependencies that would normally be included in the protocol but should 
+        be excluded for some idiosyncratic reason.
 
 Protocols are derived from the "Synthesis" and "Cleanups" columns of the 
 FreezerBox database.  There is an important distinction between these two 
@@ -162,11 +170,12 @@ not correspond to any stepwise commands, but are documented below:
 """
 
     __config__ = [
-            appcli.DocoptConfig(),
+            appcli.DocoptConfig,
     ]
 
     tags = appcli.param('<tags>')
-    include_deps = appcli.param('--no-pending', cast=not_, default=True)
+    recurse_deps = appcli.param('--no-recurse', cast=not_, default=True)
+    exclude_deps = appcli.param('--exclude', cast=lambda x: x.split(','), default=frozenset())
 
     def __init__(self, db, tags=None):
         self.db = db
@@ -176,7 +185,8 @@ not correspond to any stepwise commands, but are documented below:
         protocol = stepwise.Protocol()
         targets = iter_targets(
                 self.db, self.tags,
-                include_deps=self.include_deps,
+                recurse_deps=self.recurse_deps,
+                exclude_deps=self.exclude_deps,
         )
 
         for key, group in group_by_synthesis(targets):
@@ -249,7 +259,7 @@ class StepwiseMaker:
         else:
             cwd = getcwd()
 
-        load_cmd = ' '.join(args.by_index[1:])
+        load_cmd = shlex.join(args.by_index[1:])
         if not load_cmd:
             raise QueryError("no stepwise command specified", culprit=product)
 
@@ -263,17 +273,24 @@ class StepwiseMaker:
     def get_product_seqs(self):
         return self._product_seqs
 
-    @only_raise(QueryError)
+    @only_raise(QueryError, AttributeError)
     def get_product_conc(self):
-        return unanimous(self._product_concs)
+        return self._unanimous_or_undef(self._product_concs, 'product_conc')
 
-    @only_raise(QueryError)
+    @only_raise(QueryError, AttributeError)
     def get_product_volume(self):
-        return unanimous(self._product_volumes)
+        return self._unanimous_or_undef(self._product_volumes, 'product_volume')
 
-    @only_raise(QueryError)
+    @only_raise(QueryError, AttributeError)
     def get_product_molecule(self):
-        return unanimous(self._product_molecules)
+        return self._unanimous_or_undef(self._product_molecules, 'product_molecule')
+
+    def _unanimous_or_undef(self, values, attr):
+        return unanimous(
+                values,
+                err_empty=AttributeError(f"{self!r} object has no attribute {attr!r}"),
+                err_multiple=lambda x1, x2: QueryError(f"{self!r} object has multiple values for {attr!r}: {x1!r}, {x2!r}")
+        )
 
 
 @autoprop
@@ -313,20 +330,24 @@ def iter_makers(db, key, targets):
     factory = load_maker_factory(key)
     yield from factory(db, targets)
 
-def iter_targets(db, tags, include_deps=True):
+def iter_targets(db, tags, recurse_deps=True, exclude_deps=frozenset()):
     for tag in tags:
+        if tag in exclude_deps:
+            continue
+
         target = db[tag]
         yield target
 
-        if include_deps:
+        if recurse_deps:
             try:
                 dep_tags = target.dependencies
             except QueryError:
                 continue
             else:
                 yield from iter_targets(
-                        db, filter(lambda x: db[x].pending, dep_tags),
-                        include_deps=include_deps,
+                        db, filter(lambda x: not db[x].ready, dep_tags),
+                        recurse_deps=recurse_deps,
+                        exclude_deps=exclude_deps,
                 )
 
 def label_products(products):
